@@ -5,7 +5,7 @@
 
 hz2bark <- function(f){
     
-    if(!is.numeric(f) || f < 0)
+    if(!is.numeric(f) || any(f < 0))
       stop("frequencies have to be non-negative")
 
     # Inverse of Hynek's formula (see bark2hz)
