@@ -5,7 +5,7 @@
 
 mel2hz <- function(z, htk=FALSE){
     
-    if(!is.numeric(z) || z < 0)
+    if(!is.numeric(z) || any(z < 0))
       stop("frequencies have to be non-negative")
 
     if(htk){
